from cherrypy import cpg
from cherrypy.lib import httptools
from cherrytemplate import renderTemplate

from sqlobject import *

from utils import *

from index import WebRoot
from projects import WebProject

import crypt

from releases import Release

from datetime import date

from connection import conn

class Admin(SQLObject):
    _connection = conn
    _defaultOrder = "username"
    username = StringCol()
    password = StringCol()
    fullname = StringCol()
    email = StringCol()
    gender = StringCol()
    birthyear = IntCol()
    description = StringCol()
    homepage = StringCol()
    

class WebAdmin(WebRoot):
    def __init__(self):
        self.projects = WebProject()

    def default(self, aid):
        return self.show(aid)

    default.exposed = True


    def show(self, aid):
        d = date(2000, 1, 1)
        d = d.today()
        cur_year = d.year
            
        admin = Admin.get(aid)
        html = renderTemplate(file="templates/admin_show.html")
        return self.render(html)

    show.exposed = True

    def delete(self, aid, confirmed=None):
        admin = Admin.get(aid)
        if confirmed:
            Admin.delete(aid)
            httptools.redirect("list")
        else:
            conf_msg="Do you really want to delete administrator %s ?" % admin.username
            conf_yes="delete"
            conf_hidden = { "aid" : aid,
                            "confirmed" : "yes" }

            html = renderTemplate(file="templates/confirm_action.html")
            return self.render(html)
            
    delete = adminOnly(delete)

    def edit(self, aid, username=None, password="", email=None, fullname=None, gender="", birthyear=0, description="", homepage=""):
        admin = Admin.get(aid)
        if username and email and fullname and gender and birthyear and description and homepage:
            admin.username = username
            admin.email = email
            admin.fullname = fullname
            admin.gender = gender
            admin.birthyear = birthyear
            admin.description = description
            admin.homepage = homepage

            if password != "":
                admin.password = crypt.crypt(password, password[:2])
            
            httptools.redirect("/admin/list")
        else:
            html = renderTemplate(file="templates/admin_edit.html")
            return self.render(html)
        
    edit = adminOnly(edit)
            

    def add(self, username=None, password=None, email=None, fullname=None, gender="", birthyear=0, description="", homepage=""):
        if username and password and email and fullname:
            enc = crypt.crypt(password, password[:2])
            Admin(username=username,
                  password=enc,
                  email=email,
                  fullname=fullname,
                  gender = gender,
                  birthyear = birthyear,
                  description = description,
                  homepage = homepage
                  )

            httptools.redirect("/admin")
            
        else:
            return self.render_page("admin_add")

    add = adminOnly(add)

    def list(self):
        Admins = Admin.select()
        html = renderTemplate(file="templates/admins_list.html")
        return self.render(html)

    list = adminOnly(list)

    
    def login(self, username, passwd):
        candidate = Admin.selectBy(username=username)

        if candidate.count() > 0 and username and passwd:
            enc = crypt.crypt(passwd, passwd[:2])

            candidate = candidate[0]
            if enc == candidate.password:
                toSession(admin=True)
                toSession(username=candidate.username)
                toSession(email=candidate.email)
                toSession(fullname=candidate.fullname)


        httptools.redirect("/admin")

    login.exposed = True


    def logout(self):
        toSession(admin=False)

        httptools.redirect("/")
        
    logout = adminOnly(logout)
    
    def index(self):
        if fromSession('admin') == True:
            return self.render_page("admin_index")
        else:
            return self.render_page("admin_login")
    index.exposed = True

    def statistics(self):
        releases = Release.select(orderBy="-downloads")
        html = renderTemplate(file="templates/admin_stats.html")
        return self.render(html)
    
    statistics = adminOnly(statistics)
        
