# FileUploadFilter
#
# Author: Michal Zylinski, michal.zylinski@gmail.com
#
# Version: 1.0
#


from cherrypy import cpg
from cherrypy.lib.filter.basefilter import BaseInputFilter 
import StringIO
import cgi
import guid

class FileUploadFilter(BaseInputFilter):
	def __init__(self, pParsingContext):
		"""
		ParsingContext = [(method, param in method, directory)]
		"""
		self.ParsingContext = pParsingContext
		self.FilteringEnabled = False
		self.UploadParam = None
		self.UploadDirectory = None

	def afterRequestHeader(self):
		URLPath = cpg.request.path
		Method = URLPath[URLPath.rfind('/')+1:]
		if Method in [Item[0] for Item in self.ParsingContext]:
			cpg.request.parsePostData = False
			self.UploadParam, self.UploadDirectory = [(Item[1], Item[2]) for Item in self.ParsingContext if Item[0] == Method][0]
			self.FilteringEnabled = True
		else:
			self.FilteringEnabled = False
			self.UploadParam = None
			self.UploadDirectory = None
			
	def afterRequestBody(self):
		if self.FilteringEnabled is False: return
		# Read request body and put it in data
		len = int(cpg.request.headerMap.get("Content-Length","0"))
		if len: data = cpg.request.rfile.read(len)
		else: data=""
		newRfile = StringIO.StringIO(data)
		lowerHeaderMap = {}
		for key, value in cpg.request.headerMap.items():
				lowerHeaderMap[key.lower()] = value
		forms = cgi.FieldStorage(fp = newRfile, headers = lowerHeaderMap, environ = {'REQUEST_METHOD':'POST'}, keep_blank_values = 1)
		for key in forms.keys():
			valueList = forms[key]
			if type(valueList) == type([]):
					cpg.request.paramMap[key] = []
					cpg.request.filenameMap[key] = []
					cpg.request.fileTypeMap[key] = []
					for item in valueList:
							cpg.request.paramMap[key].append(item.value)
							cpg.request.filenameMap[key].append(item.filename)
							cpg.request.fileTypeMap[key].append(item.type)
			else: #here comes file processing routine
				if valueList.name == self.UploadParam:
					OriginalFileExtension = valueList.filename[valueList.filename.rfind('.'):]
					NewName = self.UploadDirectory+guid.generate()+OriginalFileExtension
					SaveFile = file(NewName, 'wb')          
					while 1:
						data = valueList.file.read(1024 * 8) # Read blocks of 8KB at a time
						if not data: break
						SaveFile.write(data)
					SaveFile.close()
					cpg.request.paramMap[key] = NewName
					cpg.request.filenameMap[key] = valueList.filename
					cpg.request.fileTypeMap[key] = valueList.type		
				else:
					cpg.request.paramMap[key] = valueList.value
					cpg.request.filenameMap[key] = valueList.filename
					cpg.request.fileTypeMap[key] = valueList.type


if __name__ == '__main__':	
#running test 				
	from cherrypy import cpg
	class Test:
		TempDir = '/tmp/'
		UploadContext = [('processUpload', 'fileob', TempDir)]
		_cpFilterList = [FileUploadFilter(UploadContext)]
		
		
		def index(self):
			return """<HTML><BODY>
			<FORM ACTION="processUpload" METHOD="POST" ENCTYPE="multipart/form-data">
			<INPUT TYPE="file" name="fileob">
			<INPUT TYPE="submit">
			</FORM>
			</BODY></HTML>
			"""
		index.exposed = True
		
		
		def processUpload(self, fileob):
			yield 'File Name: %s<br>' % cpg.request.filenameMap['fileob']
			yield 'File Type: %s<br>' % cpg.request.fileTypeMap['fileob']
			yield 'Stored as: %s' % fileob
			
		processUpload.exposed = True
			
	cpg.root = Test()
	cpg.server.start()
