from cherrytemplate import renderTemplate
from sqlobject import *
from cherrypy import *
from cherrypy.lib import httptools

from datetime import datetime

from utils import *
from index import WebRoot

from connection import conn

class News(SQLObject):
    _connection = conn
    _defaultOrder = "-id"
    project = ForeignKey("Project")
    author = StringCol()
    date = DateCol()
    subject = StringCol()
    content = StringCol()


class WebNews(WebRoot):
    def index(self):
        return self.list()
    index.exposed = True
    
    def list(self, pjid=None, embed=False, limit=20):
        if pjid:
            news = News.selectBy(projectID=pjid)
        else:
            news = News.select()
            news = news[:limit]

        html = renderTemplate(file="templates/news_list.html")
        if embed:
            return html
        else:
            return self.render(html)
    list.exposed = True

    def edit(self, nid, author=None, subject=None, content=None):
        news = News.get(nid)
        if author and subject and content:
            news.author = author
            news.subject = subject
            news.content = content

            httptools.redirect("/news")
        else:
            html = renderTemplate(file="templates/admin_news_edit.html")
            return self.render(html)

    edit = adminOnly(edit)

    def add(self, pjid, author=None, subject=None, content=None):
        if author and subject and content:
            d = datetime(2000, 1, 1)
            d = d.now()
            News(projectID=pjid,
                 author=author,
                 date=d,
                 subject=subject,
                 content=content)

            return self.list()
                 
        else:
            html = renderTemplate(file="templates/admin_news_add.html")
            html += self.list(pjid, True)
            
            
            return self.render(html)
        
    add = adminOnly(add)
    
    def delete(self, nid):
        News.delete(nid)
        httptools.redirect("/news")
        
    delete = adminOnly(delete)


    
