from cherrypy import cpg
from cherrytemplate import renderTemplate
from cherrypy.lib.filter import basefilter
from fileuploadfilter import FileUploadFilter
from cherrypy.lib import httptools

import cgi

from sqlobject import *

from utils import *

from index import WebRoot

from news import News, WebNews
from screenshot import Screenshot, WebScreenshots
from repositories import Repository, WebRepositories
from releases import Release, WebReleases

from connection import conn

class Project(SQLObject):
    _connection = conn
    _defaultOrder = "name" 
    name = StringCol()
    authors = StringCol()
    license = StringCol()
    homepage = StringCol()
    description = StringCol()
    vcs = StringCol()

    def html(self):
        try:
            last_rel = self.releases[0]
        except:
            last_rel = None
        
        html = renderTemplate(file="templates/projects_view.html")
        return html

    def _get_screenshots(self):
        screenies = Screenshot.selectBy(projectID=self.id)
        return screenies

    def _get_releases(self):
        releases = Release.selectBy(projectID=self.id)
        return releases
    
    def _get_repositories(self):
        repositories = Repository.selectBy(projectID=self.id)
        return repositories


class WebProject(WebRoot):
    def __init__(self):
        self.screenshots = WebScreenshots()
        self.releases = WebReleases()
        self.news = WebNews()
        self.repositories = WebRepositories()
        
    def index(self):
        return self.list()
    index.exposed = True

    def list(self):
        Projects = []
        for prj in Project.select():
            Projects.append(prj)

        html = renderTemplate(file="templates/projects_list.html")
        
        return self.render(html)
    list.exposed = True

    def edit(self, pjid, name=None, authors=None, license=None, description=None, vcs="", homepage=""):
        p = Project.get(pjid)
        if name and authors and license and description:
            p.name = name
            p.authors = authors
            p.license = license
            p.description = description
            p.homepage = homepage
            p.vcs = vcs

            httptools.redirect("/projects/%s"%p.name)
        else:
            html = renderTemplate(file="templates/admin_projects_edit.html")
            return self.render(html)

    edit = adminOnly(edit)
 

    def add(self, name=None, authors=None, license=None, description=None, vcs="", homepage=""):
        if name and authors and license and description:
            Project(name=name,
                    authors=authors,
                    license=license,
                    homepage=homepage,
                    description=description,
                    vcs=vcs)
            
            return self.list()
        else:
            return self.render_page("admin_projects_add")
    add = adminOnly(add)

    def delete(self, pjid, confirmed=None):
        prj = Project.get(pjid)

        if confirmed:
            for scr in prj.screenshots:
                Screenshot.delete(scr.id)

            for rel in prj.releases:
                Release.delete(rel.id)

            news = News.selectBy(projectID=pjid)
            for n in news:
                News.delete(n.id)
            
            Project.delete(pjid)

            return self.list()
            
        else:
            conf_msg="Do you really want to delete project %s ?" % prj.name
            conf_yes="delete"
            conf_hidden = { "pjid" : pjid,
                            "confirmed" : "yes" }
            
            html = renderTemplate(file="templates/confirm_action.html")
            return self.render(html)
    
    delete = adminOnly(delete)

    def default(self, name):
        return self.view(name)
    default.exposed = True
    
    def view(self, name):
        name = name.replace("%20", " ")
        # try:
        MyProject = Project.selectBy(name=name)[0]
        html = MyProject.html()
        #except:
        #    html = "No such project"

        return self.render(html)

