from cherrypy import cpg
from cherrypy.lib.filter import basefilter
from cherrytemplate import renderTemplate
from fileuploadfilter import FileUploadFilter
from cherrypy.lib import httptools

import shutil, os
from datetime import datetime
from sqlobject import *

from index import WebRoot
from news import News

from utils import *

from connection import conn

class Release(SQLObject):
    _connection = conn
    _defaultOrder="-id"
    project = ForeignKey("Project")
    filename = StringCol()
    tag = StringCol()
    changelog = StringCol()
    downloads = IntCol()

class WebReleases(WebRoot):
    UploadContext = [('upload', 'filename', 'uploads/'),
                     ('edit', 'filename', 'uploads/')]
    _cpFilterList = [FileUploadFilter(UploadContext)]


    def upload(self, pjid,  filename, tag, changelog, news=False):
        #yield 'File Name: %s<br>' % cpg.request.filenameMap['filename']
        #yield 'File Type: %s<br>' % cpg.request.fileTypeMap['filename']
        #yield 'Stored as: %s' % filename

        dstname = cpg.request.filenameMap['filename']

        basepath = "releases/%d" % int(pjid)
        if not os.path.exists(basepath):
            os.mkdir(basepath)
            
        shutil.move(filename, "%s/%s"%(basepath, dstname))
        self.add(pjid, dstname, tag, changelog)

        # Add news !
        if news:
            content = """This project has been updated to version %s.<br>
            This new version features the following changes :<br>
            %s
            <br>
            <br>
            You can download it <a href="/releases/%d/%s">here</a>.""" % (tag, changelog, int(pjid), dstname)
            d = datetime(2000, 1, 1)
            d = d.now()
        
            News(projectID=pjid,
                 author="patapouf website",
                 subject="New release !",
                 content=content,
                 date=d)
        
        return self.list(pjid)

    upload = adminOnly(upload)

    def add(self, pjid, filename, tag, changelog):
        Release(projectID=pjid,
                filename=filename,
                tag=tag,
                changelog=changelog,
                downloads=0)

    def edit(self, rid, filename=None, tag=None, changelog=None):
        release = Release.get(rid)

        if tag and changelog:
            if cpg.request.filenameMap["filename"] != "":
                dstname = cpg.request.filenameMap['filename']
                basepath = "releases/%d" % release.project.id
                oldpath = os.path.join(basepath, release.filename)
                newpath = os.path.join(basepath, dstname)
                
                os.unlink(oldpath)
                shutil.move(filename, newpath)

                release.filename = dstname
            elif filename != None:
                os.unlink(filename)

            release.tag = tag
            release.changelog = changelog
            
            httptools.redirect("/projects/%s"%release.project.name)
        else:
            if filename != None:
                os.unlink(filename)
            html = renderTemplate(file="templates/admin_releases_edit.html")
            return self.render(html)

    edit = adminOnly(edit)
    
    def delete(self, rid, ref=None):
        r = Release.get(rid)
        try:
            Release.delete(rid)
            os.unlink("releases/%d/%s"%(int(r.projectID), r.filename))
            # FIXME: If dir is empty, delete
        except:
            pass

        if not ref:
            return self.list(r.projectID)
        else:
            httptools.redirect(ref)
        
    delete = adminOnly(delete)

    def list(self, pjid):
        Releases = Release.selectBy(projectID=pjid)
        html = renderTemplate(file="templates/releases_list.html")
        return self.render(html)

    list = adminOnly(list)

