from cherrypy import cpg
from cherrytemplate import renderTemplate
from cherrypy.lib.filter import basefilter
from fileuploadfilter import FileUploadFilter

from cherrypy.lib import httptools

import Image

import shutil, os

from sqlobject import *
from utils import *

from index import WebRoot

from connection import conn

class Screenshot(SQLObject):
    _connection = conn
    _defaultOrder = "-id"
    project = ForeignKey("Project")
    filename = StringCol()
    description = StringCol()
    
    
class WebScreenshots(WebRoot):
    UploadContext = [('upload', 'filename', 'uploads/'),
                     ('edit', 'filename', 'uploads/')]
    _cpFilterList = [FileUploadFilter(UploadContext)]

    def upload(self, pjid,  filename, description):
        #yield 'File Name: %s<br>' % cpg.request.filenameMap['filename']
        #yield 'File Type: %s<br>' % cpg.request.fileTypeMap['filename']
        #yield 'Stored as: %s' % filename

        dstname = filename.split("/")[-1]
        dstname = "%d_%s"%(int(pjid), dstname)
        shutil.move(filename, "screenshots/%s"%dstname)
        
        self.make_preview("screenshots/%s"%dstname, 250)
        
        self.add(pjid, dstname, description)
        
        return self.list(pjid)

    upload = adminOnly(upload)

    def make_preview(self, filename, thumb_size):
        im = Image.open(filename)
        
        if im.size[0] > im.size[1]:
            w = thumb_size
            h = (w*im.size[1])/im.size[0]
        else:
            h = thumb_size
            w = (h*im.size[0])/im.size[1]

        im.thumbnail((w, h), Image.ANTIALIAS)
        im.save(filename.split("/")[0] + "/prev_"+filename.split("/")[1], im.format)
            
    def add(self, pjid, filename, description):
        Screenshot(projectID=pjid,
                   filename=filename,
                   description=description)

    def edit(self, sid, filename=None, description=None):
        screenshot = Screenshot.get(sid)

        if description:
            if cpg.request.filenameMap["filename"] != "":
                shutil.move(filename, "screenshots/%s"%screenshot.filename)
                self.make_preview("screenshots/%s"%screenshot.filename, 250)
            elif filename != None:
                os.unlink(filename)
                
            screenshot.description = description
            httptools.redirect("/projects/%s"%screenshot.project.name)
        else:
            if filename != None:
                os.unlink(filename)
            html = renderTemplate(file="templates/admin_screenshots_edit.html")
            return self.render(html)

    edit = adminOnly(edit)
        
    def delete(self, sid, ref=None):
        s = Screenshot.get(sid)
        try:
            Screenshot.delete(sid)
            os.unlink("screenshots/"+s.filename)
            os.unlink("screenshots/prev_"+s.filename)
        except:
            pass

        if not ref:
            return self.list(s.projectID)
        else:
            httptools.redirect(ref)
        
    delete = adminOnly(delete)

    def list(self, pjid):
        
        Screenshots = Screenshot.selectBy(projectID=pjid)
        html = renderTemplate(file="templates/screenshots_list.html")
        return self.render(html)

    list = adminOnly(list)

