from cherrypy import cpg
from cherrypy.cperror import *



def toSession(**kwp):
    """Stores the keyword parameters in the cpg.request.sessionMap.
    like:
    toSession(bla=bla) will store the variable bla as 'bla'
    @returns: nothing.    
    """
    for k,v in kwp.items():
        cpg.request.sessionMap[k] = v

def fromSession(*names):
    """Retrives the names from the cpg.request.sessionMap.
    like:
    bla,pir = fromSession('bla','pir') will load bla and pir
    @returns: the values for the keys named in 'names'
    @raises: nothing, None as value for a key that can't be found
    """
    values = []
    for name in names:
        try:
            values.append(cpg.request.sessionMap.get(name,None))
        except AttributeError:
            values.append(None)
    if len(values) ==1:
        # allow for easy element retreival
        return values[0]
    else:
        return values

def checkAuth():
    # this is called default before the actual method is called, 
    # when a method is exposed using
    #    methodname = expose(methodname)
    # one could add basic checking in here
    pass

def expose(func, preFunc=checkAuth, postFunc=None):
    def helper(*p,**kwp):
        if preFunc:
            preFunc()
        result = func(*p,**kwp)
        if postFunc:
            postFunc()
        return result   
    helper.exposed = True
    return helper

def adminOnly(func):
    def checkIsAdminOrRaiseError():
        if fromSession('admin') != True:
            raise NotFound
    return expose(func,preFunc = checkIsAdminOrRaiseError)
    
def validCustomerOnly(func):
    def checkIsValidUserOrRaiseError():
        # check if it's a valid customer number
        # see this as sample code :))
        if fromSession('user') not in cpg.root.dataprovider.debtors:
            raise NotFound
        return expose(func,preFunc = checkIsValidUserOrRaiseError)


def is_admin():
    return fromSession('admin') == True


def dev_link(cont):
    from admin import Admin
    from random import Random

    # Link all admins names to mail

    Content = cont
    admins = Admin.select()
    for admin in admins:
        r = Random()

        rep = """<a href="/developer/%d">%s</a>""" \
              % (admin.id,
                 admin.username)

        Content = Content.replace(admin.username, rep)
            
        rep = """<a href="/developer/%d">%s</a>""" \
              % (admin.id,
                 admin.fullname)
        Content = Content.replace(admin.fullname, rep)

    return Content
    

def mail_link(cont):
    from admin import Admin
    from random import Random

    # Link all admins names to mail

    Content = cont
    admins = Admin.select()
    for admin in admins:
        r = Random()

        rep = """<a href="mailto:%s_%d(AT)%d_%s">%s</a>""" \
              % (admin.email.split("@")[0],
                 r.randint(1, 10),
                 r.randint(1, 10),
                 admin.email.split("@")[1],
                 "[mail link]")

        Content = Content.replace(admin.email, rep)
            
    return Content

def distrib_logo(dist):
    return """<img class="button" src="/images/%s.png" alt="%s">"""%(dist.lower(), dist)

def syntaxize(txt):
    replaces = { "[FR]" : """<img src="/images/fr.png" alt="FR">""",
                 "[EN]" : """<img src="/images/en.png" alt="EN">"""
                 }

    for el in replaces.keys():
        k,v = el, replaces.get(el)
        txt = txt.replace(k, v)

    return txt

def link_vcs(txt):
    if txt.find("vcs.patapouf.org/arch/"):
        txt = 'Arch : <a href="http://vcs.patapouf.org/cgi-bin/archzoom.cgi/devel@patapouf.org--patapouf-arch/%s?expand">%s</a> (RO Anonymous access).' % (txt.rstrip("/").split("/")[-1].split("--")[0], txt)
    return txt
